"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
function control_changes(sts) {
    function identify_change(key) {
        var val = sts.get(key);
        var val_backup = sts.backup.get(key);
        if (Array.isArray(val) && Array.isArray(val_backup) && val.length !== val_backup.length) {
            return true;
        }
        else if (Array.isArray(val) || Array.isArray(val_backup)) {
            return JSON.stringify(val) !== JSON.stringify(val_backup);
        }
        else if (typeof val === 'object' || typeof val_backup === 'object') {
            return JSON.stringify(val) !== JSON.stringify(val_backup);
        }
        else if (utils_1.isSttore(val)) {
            return val.change();
        }
        return val !== val_backup;
    }
    function change(key) {
        if (key !== undefined) {
            if (!sts.has(key))
                return false;
            return identify_change(key);
        }
        var has_change = false;
        for (var key_1 in sts.list()) {
            has_change = identify_change(key_1);
            if (has_change)
                break;
        }
        return has_change;
    }
    function changes() {
        var changes = {};
        var list = sts.list();
        for (var key in list) {
            var value = list[key];
            if (identify_change(key)) {
                changes[key] = value;
            }
        }
        return changes;
    }
    function omit(keys) {
        var has_change = false;
        var _loop_1 = function (key) {
            if (keys.find(function (_k) { return _k === key; }))
                return "continue";
            if (has_change)
                return "break";
            has_change = change(key);
        };
        for (var key in sts.list()) {
            var state_1 = _loop_1(key);
            if (state_1 === "break")
                break;
        }
        return has_change;
    }
    function only(keys) {
        var has_change = false;
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            if (has_change)
                break;
            has_change = change(key);
        }
        return has_change;
    }
    return {
        change: change,
        changes: changes,
        omit: omit,
        only: only
    };
}
exports.default = control_changes;
